@@ -4,6 +4,10 @@ import smbus
 
 # ===========================================================================
 # Adafruit_I2C Class
+# Adafruit_I2C.py is essentially a fork of the Adafruit Raspberry Pi I2C module.
+# Any pull requests for this module should be directed to the following, and I
+# can pull them. I'd rather not deviate from the original:
+# https://github.com/adafruit/Adafruit-Raspberry-Pi-Python-Code/tree/master/Adafruit_I2C
 # ===========================================================================
 
 class Adafruit_I2C :
@@ -24,7 +28,7 @@ class Adafruit_I2C :
     return val
 
   def errMsg(self):
-    print "Error accessing 0x%02X: Check your I2C address" % self.address
+    print("Error accessing 0x%02X: Check your I2C address" % self.address)
     return -1
 
   def write8(self, reg, value):
@@ -32,8 +36,8 @@ class Adafruit_I2C :
     try:
       self.bus.write_byte_data(self.address, reg, value)
       if self.debug:
-        print "I2C: Wrote 0x%02X to register 0x%02X" % (value, reg)
-    except IOError, err:
+        print("I2C: Wrote 0x%02X to register 0x%02X" % (value, reg))
+    except IOError as err:
       return self.errMsg()
 
   def write16(self, reg, value):
@@ -41,19 +45,19 @@ class Adafruit_I2C :
     try:
       self.bus.write_word_data(self.address, reg, value)
       if self.debug:
-        print ("I2C: Wrote 0x%02X to register pair 0x%02X,0x%02X" %
+        print("I2C: Wrote 0x%02X to register pair 0x%02X,0x%02X" %
          (value, reg, reg+1))
-    except IOError, err:
+    except IOError as err:
       return self.errMsg()
 
   def writeList(self, reg, list):
     "Writes an array of bytes using I2C format"
     try:
       if self.debug:
-        print "I2C: Writing list to register 0x%02X:" % reg
-        print list
+        print("I2C: Writing list to register 0x%02X:" % reg)
+        print(list)
       self.bus.write_i2c_block_data(self.address, reg, list)
-    except IOError, err:
+    except IOError as err:
       return self.errMsg()
 
   def readList(self, reg, length):
@@ -61,11 +65,11 @@ class Adafruit_I2C :
     try:
       results = self.bus.read_i2c_block_data(self.address, reg, length)
       if self.debug:
-        print ("I2C: Device 0x%02X returned the following from reg 0x%02X" %
+        print("I2C: Device 0x%02X returned the following from reg 0x%02X" %
          (self.address, reg))
-        print results
+        print(results)
       return results
-    except IOError, err:
+    except IOError as err:
       return self.errMsg()
 
   def readU8(self, reg):
@@ -73,10 +77,10 @@ class Adafruit_I2C :
     try:
       result = self.bus.read_byte_data(self.address, reg)
       if self.debug:
-        print ("I2C: Device 0x%02X returned 0x%02X from reg 0x%02X" %
+        print("I2C: Device 0x%02X returned 0x%02X from reg 0x%02X" %
          (self.address, result & 0xFF, reg))
       return result
-    except IOError, err:
+    except IOError as err:
       return self.errMsg()
 
   def readS8(self, reg):
@@ -85,10 +89,10 @@ class Adafruit_I2C :
       result = self.bus.read_byte_data(self.address, reg)
       if result > 127: result -= 256
       if self.debug:
-        print ("I2C: Device 0x%02X returned 0x%02X from reg 0x%02X" %
+        print("I2C: Device 0x%02X returned 0x%02X from reg 0x%02X" %
          (self.address, result & 0xFF, reg))
       return result
-    except IOError, err:
+    except IOError as err:
       return self.errMsg()
 
   def readU16(self, reg):
@@ -96,9 +100,9 @@ class Adafruit_I2C :
     try:
       result = self.bus.read_word_data(self.address,reg)
       if (self.debug):
-        print "I2C: Device 0x%02X returned 0x%04X from reg 0x%02X" % (self.address, result & 0xFFFF, reg)
+        print("I2C: Device 0x%02X returned 0x%04X from reg 0x%02X" % (self.address, result & 0xFFFF, reg))
       return result
-    except IOError, err:
+    except IOError as err:
       return self.errMsg()
 
   def readS16(self, reg):
@@ -106,14 +110,14 @@ class Adafruit_I2C :
     try:
       result = self.bus.read_word_data(self.address,reg)
       if (self.debug):
-        print "I2C: Device 0x%02X returned 0x%04X from reg 0x%02X" % (self.address, result & 0xFFFF, reg)
+        print("I2C: Device 0x%02X returned 0x%04X from reg 0x%02X" % (self.address, result & 0xFFFF, reg))
       return result
-    except IOError, err:
+    except IOError as err:
       return self.errMsg()
 
 if __name__ == '__main__':
   try:
     bus = Adafruit_I2C(address=0)
-    print "Default I2C bus is accessible"
+    print("Default I2C bus is accessible")
   except:
-    print "Error accessing default I2C bus"
+    print("Error accessing default I2C bus")
