@@ -1,3 +1,6 @@
+#ifndef CONSTANTS_H
+#define CONSTANTS_H
+
 PyObject *high;
 PyObject *low;
 PyObject *input;
@@ -11,4 +14,6 @@ PyObject *falling_edge;
 PyObject *both_edge;
 PyObject *version;
 
-void define_constants(PyObject *module);
\ No newline at end of file
+void define_constants(PyObject *module);
+
+#endif
