@@ -26,30 +26,42 @@ SOFTWARE.
 #include "common.h"
 #include "c_adc.h"
 
+// Cleanup function commented out because it is currently disabled and this
+// generates unused method warnings. See adc_methods[] definition further 
+// below for reason for disabling the cleanup function.
 // python function cleanup()
-static PyObject *py_cleanup(PyObject *self, PyObject *args)
+//static PyObject *py_cleanup(PyObject *self, PyObject *args)
+//{
+//    // unexport the ADC
+//    adc_cleanup();
+//
+//    Py_RETURN_NONE;
+//}
+
+// python function setup()
+static PyObject *py_setup_adc(__attribute__ ((unused)) PyObject *self, __attribute__ ((unused)) PyObject *args)
 {
-    // unexport the ADC
-    adc_cleanup();
+    BBIO_err err;
+
+    err = adc_setup();
+    if (err != BBIO_OK) {
+        PyErr_SetString(PyExc_RuntimeError, "Unable to setup ADC system. Possible causes are: \n"
+                                        "  - A cape with a conflicting pin mapping is loaded \n"
+                                        "  - A device tree object is loaded that uses the same name for a fragment: helper");
+        return NULL;
+    }
 
     Py_RETURN_NONE;
 }
 
-// python function setup()
-static PyObject *py_setup_adc(PyObject *self, PyObject *args)
-{
-    if (adc_setup())
-        Py_RETURN_NONE;
-    return NULL;
-}
-
 // python function read(channel)
-static PyObject *py_read(PyObject *self, PyObject *args)
+static PyObject *py_read(__attribute__ ((unused)) PyObject *self, PyObject *args)
 {
-    unsigned int ain;
+    int ain;
     float value;
     char *channel;
     PyObject *py_value;
+    BBIO_err err;
 
     if (!PyArg_ParseTuple(args, "s", &channel))
         return NULL;
@@ -61,15 +73,24 @@ static PyObject *py_read(PyObject *self, PyObject *args)
         return NULL;
     }    
 
-    if (!get_adc_ain(channel, &ain)) {
+    err = get_adc_ain(channel, &ain);
+    if (err != BBIO_OK) {
         PyErr_SetString(PyExc_ValueError, "Invalid AIN key or name.");
         return NULL;    
     }
 
-    read_value(ain, &value);
+    err = read_value(ain, &value);
+    if (err != BBIO_OK) {
+        PyErr_SetFromErrnoWithFilename(PyExc_IOError, "Error while reading AIN port. Invalid or locked AIN file.");
+        return NULL;
+    }
 
     //scale modifier
+#ifdef BBBVERSION41
+    value = value / 4095.0;
+#else
     value = value / 1800.0;
+#endif
 
     py_value = Py_BuildValue("f", value);
 
@@ -77,12 +98,13 @@ static PyObject *py_read(PyObject *self, PyObject *args)
 }
 
 // python function read(channel)
-static PyObject *py_read_raw(PyObject *self, PyObject *args)
+static PyObject *py_read_raw(__attribute__ ((unused)) PyObject *self, PyObject *args)
 {
-    unsigned int ain;
+    int ain;
     float value;
     char *channel;
     PyObject *py_value;
+    BBIO_err err;
 
     if (!PyArg_ParseTuple(args, "s", &channel))
         return NULL;
@@ -94,12 +116,18 @@ static PyObject *py_read_raw(PyObject *self, PyObject *args)
         return NULL;
     }       
 
-    if (!get_adc_ain(channel, &ain)) {
+    err = get_adc_ain(channel, &ain);
+    if (err != BBIO_OK) {
         PyErr_SetString(PyExc_ValueError, "Invalid AIN key or name.");
         return NULL;    
     }
 
-    read_value(ain, &value);
+    err = read_value(ain, &value);
+
+    if (err != BBIO_OK) {
+        PyErr_SetFromErrnoWithFilename(PyExc_IOError, "Error while reading AIN port. Invalid or locked AIN file.");
+        return NULL;
+    }
 
     py_value = Py_BuildValue("f", value);
 
@@ -152,4 +180,4 @@ PyMODINIT_FUNC initADC(void)
 #else
     return;
 #endif
-}
\ No newline at end of file
+}
