@@ -4,7 +4,7 @@ Copyright (c) 2013 Adafruit
 Original RPi.GPIO Author Ben Croston
 Modified for BBIO Author Justin Cooper
 
-This file incorporates work covered by the following copyright and 
+This file incorporates work covered by the following copyright and
 permission notice, all modified code adopts the original license:
 
 Copyright (c) 2013 Ben Croston
@@ -32,8 +32,10 @@ SOFTWARE.
 #include "constants.h"
 #include "common.h"
 #include "event_gpio.h"
+#include "c_pinmux.h"
+#include <unistd.h>
+#include <syslog.h>
 
-static PyObject *bb_revision;
 static int gpio_warnings = 1;
 
 struct py_callback
@@ -60,7 +62,7 @@ static int init_module(void)
 }
 
 // python function cleanup()
-static PyObject *py_cleanup(PyObject *self, PyObject *args)
+static PyObject *py_cleanup(__attribute__ ((unused)) PyObject *self, __attribute__ ((unused)) PyObject *args)
 {
     // clean up any exports
     event_cleanup();
@@ -68,23 +70,28 @@ static PyObject *py_cleanup(PyObject *self, PyObject *args)
     Py_RETURN_NONE;
 }
 
-// python function setup(channel, direction, pull_up_down=PUD_OFF, initial=None)
-static PyObject *py_setup_channel(PyObject *self, PyObject *args, PyObject *kwargs)
+// python function setup(channel, direction, pull_up_down=PUD_OFF, initial=None, delay=0)
+static PyObject *py_setup_channel(__attribute__ ((unused)) PyObject *self, PyObject *args, PyObject *kwargs)
 {
    unsigned int gpio;
    char *channel;
    int direction;
    int pud = PUD_OFF;
-   int initial = -1;
-   static char *kwlist[] = {"channel", "direction", "pull_up_down", "initial", NULL};
+   int initial = 0;
+   int delay = 0; // time in milliseconds to wait after exporting gpio pin
+   BBIO_err err;
+   static char *kwlist[] = {"channel", "direction", "pull_up_down", "initial", "delay", NULL};
+   int res;
+   // char error[30];
 
-   if (!PyArg_ParseTupleAndKeywords(args, kwargs, "si|ii", kwlist, &channel, &direction, &pud, &initial))
+
+   if (!PyArg_ParseTupleAndKeywords(args, kwargs, "si|iii", kwlist, &channel, &direction, &pud, &initial, &delay))
       return NULL;
 
    if (!module_setup) {
       init_module();
    }
-   
+
 
    if (direction != INPUT && direction != OUTPUT)
    {
@@ -101,14 +108,49 @@ static PyObject *py_setup_channel(PyObject *self, PyObject *args, PyObject *kwar
       return NULL;
    }
 
-    
 
-   if (get_gpio_number(channel, &gpio))
+   err = get_gpio_number(channel, &gpio);
+   if (err != BBIO_OK)
        return NULL;
 
+   // Export the GPIO pins using sysfs
    gpio_export(gpio);
-   gpio_set_direction(gpio, direction);
-   gpio_set_value(gpio, pud);
+
+   // See if a delay was specified in order to give udev some time 
+   // to set file permissions.
+   if (delay > 0)
+       nanosleep((struct timespec[]){{0, delay * 1000000}}, NULL);
+
+   // Set the pin direction and bail if we get an error.
+   res = gpio_set_direction(gpio, direction);
+   if(res != 0) {
+       PyErr_SetString(PyExc_ValueError, "Set gpio direction failed, missing file or invalid permissions.");
+       return NULL;
+   }
+
+   if (direction == OUTPUT) {
+
+       // Set the pin value and bail if we get an error.
+       res = gpio_set_value(gpio, initial);
+       if (res != BBIO_OK) {
+           PyErr_SetString(PyExc_ValueError, "Set gpio value failed, missing file or invalid permissions.");
+           return NULL;
+       }
+
+   } else {
+       if (pud == PUD_DOWN)
+		   res = set_pin_mode(channel, "gpio_pd");
+	   else if (pud == PUD_UP)
+		   res = set_pin_mode(channel, "gpio_pu");
+	   else
+		   res = set_pin_mode(channel, "gpio");
+   }
+   
+   //Check if set_pin_mode() returned no error
+   if (res != BBIO_OK) {
+       PyErr_SetString(PyExc_ValueError, "Set gpio mode failed, missing file or invalid permissions.");
+       return NULL;
+   }
 
    gpio_direction[gpio] = direction;
 
@@ -116,17 +158,19 @@ static PyObject *py_setup_channel(PyObject *self, PyObject *args, PyObject *kwar
 }
 
 // python function output(channel, value)
-static PyObject *py_output_gpio(PyObject *self, PyObject *args)
+static PyObject *py_output_gpio(__attribute__ ((unused)) PyObject *self, PyObject *args)
 {
     unsigned int gpio;
     int value;
     char *channel;
+    BBIO_err err;
 
     if (!PyArg_ParseTuple(args, "si", &channel, &value))
         return NULL;
 
-    if (get_gpio_number(channel, &gpio))
-        return NULL;      
+    err = get_gpio_number(channel, &gpio);
+    if (err != BBIO_OK)
+        return NULL;
 
     if (!module_setup || gpio_direction[gpio] != OUTPUT)
     {
@@ -140,17 +184,19 @@ static PyObject *py_output_gpio(PyObject *self, PyObject *args)
 }
 
 // python function value = input(channel)
-static PyObject *py_input_gpio(PyObject *self, PyObject *args)
+static PyObject *py_input_gpio(__attribute__ ((unused)) PyObject *self, PyObject *args)
 {
     unsigned int gpio;
     char *channel;
     unsigned int value;
     PyObject *py_value;
+    BBIO_err err;
 
     if (!PyArg_ParseTuple(args, "s", &channel))
         return NULL;
 
-    if (get_gpio_number(channel, &gpio))
+    err = get_gpio_number(channel, &gpio);
+    if (err != BBIO_OK)
         return NULL;
 
    // check channel is set up as an input or output
@@ -182,6 +228,10 @@ static void run_py_callbacks(unsigned int gpio)
          gettimeofday(&tv_timenow, NULL);
          timenow = tv_timenow.tv_sec*1E6 + tv_timenow.tv_usec;
          if (cb->bouncetime == 0 || timenow - cb->lastcall > cb->bouncetime*1000 || cb->lastcall == 0 || cb->lastcall > timenow) {
+
+            // save lastcall before calling func to prevent reentrant bounce
+            cb->lastcall = timenow;
+
             // run callback
             gstate = PyGILState_Ensure();
             result = PyObject_CallFunction(cb->py_cb, "s", cb->channel);
@@ -233,13 +283,14 @@ static int add_py_callback(char *channel, unsigned int gpio, unsigned int bounce
 }
 
 // python function add_event_callback(gpio, callback, bouncetime=0)
-static PyObject *py_add_event_callback(PyObject *self, PyObject *args, PyObject *kwargs)
+static PyObject *py_add_event_callback(__attribute__ ((unused)) PyObject *self, PyObject *args, PyObject *kwargs)
 {
    unsigned int gpio;
    char *channel;
    unsigned int bouncetime = 0;
    PyObject *cb_func;
    char *kwlist[] = {"gpio", "callback", "bouncetime", NULL};
+   BBIO_err err;
 
    if (!PyArg_ParseTupleAndKeywords(args, kwargs, "sO|i", kwlist, &channel, &cb_func, &bouncetime))
       return NULL;
@@ -250,7 +301,8 @@ static PyObject *py_add_event_callback(PyObject *self, PyObject *args, PyObject
       return NULL;
    }
 
-   if (get_gpio_number(channel, &gpio))
+   err = get_gpio_number(channel, &gpio);
+   if (err != BBIO_OK)
        return NULL;
 
    // check channel is set up as an input
@@ -273,7 +325,7 @@ static PyObject *py_add_event_callback(PyObject *self, PyObject *args, PyObject
 }
 
 // python function add_event_detect(gpio, edge, callback=None, bouncetime=0
-static PyObject *py_add_event_detect(PyObject *self, PyObject *args, PyObject *kwargs)
+static PyObject *py_add_event_detect(__attribute__ ((unused)) PyObject *self, PyObject *args, PyObject *kwargs)
 {
    unsigned int gpio;
    char *channel;
@@ -281,6 +333,7 @@ static PyObject *py_add_event_detect(PyObject *self, PyObject *args, PyObject *k
    unsigned int bouncetime = 0;
    PyObject *cb_func = NULL;
    char *kwlist[] = {"gpio", "edge", "callback", "bouncetime", NULL};
+   BBIO_err err;
 
    if (!PyArg_ParseTupleAndKeywords(args, kwargs, "si|Oi", kwlist, &channel, &edge, &cb_func, &bouncetime))
       return NULL;
@@ -291,7 +344,8 @@ static PyObject *py_add_event_detect(PyObject *self, PyObject *args, PyObject *k
       return NULL;
    }
 
-   if (get_gpio_number(channel, &gpio))
+   err = get_gpio_number(channel, &gpio);
+   if (err != BBIO_OK)
        return NULL;
 
    // check channel is set up as an input
@@ -328,19 +382,21 @@ static PyObject *py_add_event_detect(PyObject *self, PyObject *args, PyObject *k
 }
 
 // python function remove_event_detect(gpio)
-static PyObject *py_remove_event_detect(PyObject *self, PyObject *args)
+static PyObject *py_remove_event_detect(__attribute__ ((unused)) PyObject *self, PyObject *args)
 {
    unsigned int gpio;
    char *channel;
    struct py_callback *cb = py_callbacks;
    struct py_callback *temp;
    struct py_callback *prev = NULL;
+   BBIO_err err;
 
    if (!PyArg_ParseTuple(args, "s", &channel))
       return NULL;
 
-   if (get_gpio_number(channel, &gpio))
-        return NULL;
+   err = get_gpio_number(channel, &gpio);
+   if (err != BBIO_OK)
+       return NULL;
 
    // remove all python callbacks for gpio
    while (cb != NULL)
@@ -367,15 +423,17 @@ static PyObject *py_remove_event_detect(PyObject *self, PyObject *args)
 }
 
 // python function value = event_detected(channel)
-static PyObject *py_event_detected(PyObject *self, PyObject *args)
+static PyObject *py_event_detected(__attribute__ ((unused)) PyObject *self, PyObject *args)
 {
    unsigned int gpio;
    char *channel;
+   BBIO_err err;
 
    if (!PyArg_ParseTuple(args, "s", &channel))
       return NULL;
 
-   if (get_gpio_number(channel, &gpio))
+   err = get_gpio_number(channel, &gpio);
+   if (err != BBIO_OK)
        return NULL;
 
    if (event_detected(gpio))
@@ -384,19 +442,23 @@ static PyObject *py_event_detected(PyObject *self, PyObject *args)
       Py_RETURN_FALSE;
 }
 
-// python function py_wait_for_edge(gpio, edge)
-static PyObject *py_wait_for_edge(PyObject *self, PyObject *args)
+// python function py_wait_for_edge(gpio, edge, timeout = -1)
+static PyObject *py_wait_for_edge(__attribute__ ((unused)) PyObject *self, PyObject *args)
 {
    unsigned int gpio;
-   int edge, result;
+   int edge, result, timeout;
    char *channel;
    char error[30];
+   BBIO_err err;
 
-   if (!PyArg_ParseTuple(args, "si", &channel, &edge))
+   timeout = -1;
+   if (!PyArg_ParseTuple(args, "si|i", &channel, &edge, &timeout)){
       return NULL;
+   }
 
-   if (get_gpio_number(channel, &gpio))
-      return NULL;
+   err = get_gpio_number(channel, &gpio);
+   if (err != BBIO_OK)
+       return NULL;
 
    // check channel is setup as an input
    if (!module_setup || gpio_direction[gpio] != INPUT)
@@ -413,12 +475,14 @@ static PyObject *py_wait_for_edge(PyObject *self, PyObject *args)
    }
 
    Py_BEGIN_ALLOW_THREADS // disable GIL
-   result = blocking_wait_for_edge(gpio, edge);
+   result = blocking_wait_for_edge(gpio, edge, timeout);
    Py_END_ALLOW_THREADS   // enable GIL
 
    if (result == 0) {
       Py_INCREF(Py_None);
       return Py_None;
+   }else if (result == -1){
+      Py_RETURN_FALSE;
    } else if (result == 2) {
       PyErr_SetString(PyExc_RuntimeError, "Edge detection events already enabled for this GPIO channel");
       return NULL;
@@ -427,24 +491,25 @@ static PyObject *py_wait_for_edge(PyObject *self, PyObject *args)
       PyErr_SetString(PyExc_RuntimeError, error);
       return NULL;
    }
-   
+
    Py_RETURN_NONE;
 }
 
 // python function value = gpio_function(gpio)
-static PyObject *py_gpio_function(PyObject *self, PyObject *args)
+static PyObject *py_gpio_function(__attribute__ ((unused)) PyObject *self, PyObject *args)
 {
     unsigned int gpio;
     unsigned int value;
     PyObject *func;
     char *channel;
-
+    BBIO_err err;
 
     if (!PyArg_ParseTuple(args, "s", &channel))
        return NULL;
 
-    if (get_gpio_number(channel, &gpio))
-        return NULL;
+   err = get_gpio_number(channel, &gpio);
+   if (err != BBIO_OK)
+       return NULL;
 
     if (setup_error)
     {
@@ -458,7 +523,7 @@ static PyObject *py_gpio_function(PyObject *self, PyObject *args)
 }
 
 // python function setwarnings(state)
-static PyObject *py_setwarnings(PyObject *self, PyObject *args)
+static PyObject *py_setwarnings(__attribute__ ((unused)) PyObject *self, __attribute__ ((unused)) PyObject *args)
 {
    if (!PyArg_ParseTuple(args, "i", &gpio_warnings))
       return NULL;
@@ -475,7 +540,7 @@ static PyObject *py_setwarnings(PyObject *self, PyObject *args)
 static const char moduledocstring[] = "GPIO functionality of a BeagleBone using Python";
 
 PyMethodDef gpio_methods[] = {
-   {"setup", (PyCFunction)py_setup_channel, METH_VARARGS | METH_KEYWORDS, "Set up the GPIO channel, direction and (optional) pull/up down control\nchannel        - Either: RPi board pin number (not BCM GPIO 00..nn number).  Pins start from 1\n                 or    : BCM GPIO number\ndirection      - INPUT or OUTPUT\n[pull_up_down] - PUD_OFF (default), PUD_UP or PUD_DOWN\n[initial]      - Initial value for an output channel"},
+   {"setup", (PyCFunction)py_setup_channel, METH_VARARGS | METH_KEYWORDS, "Set up the GPIO channel, direction and (optional) pull/up down control\nchannel        - Either: RPi board pin number (not BCM GPIO 00..nn number).  Pins start from 1\n                 or    : BCM GPIO number\ndirection      - INPUT or OUTPUT\n[pull_up_down] - PUD_OFF (default), PUD_UP or PUD_DOWN\n[initial]      - Initial value for an output channel\n[delay]        - Time in milliseconds to wait after exporting gpio pin"},
    {"cleanup", py_cleanup, METH_VARARGS, "Clean up by resetting all GPIO channels that have been used by this program to INPUT with no pullup/pulldown and no event detection"},
    {"output", py_output_gpio, METH_VARARGS, "Output to a GPIO channel\ngpio  - gpio channel\nvalue - 0/1 or False/True or LOW/HIGH"},
    {"input", py_input_gpio, METH_VARARGS, "Input from a GPIO channel.  Returns HIGH=1=True or LOW=0=False\ngpio - gpio channel"},
@@ -483,7 +548,7 @@ PyMethodDef gpio_methods[] = {
    {"remove_event_detect", py_remove_event_detect, METH_VARARGS, "Remove edge detection for a particular GPIO channel\ngpio - gpio channel"},
    {"event_detected", py_event_detected, METH_VARARGS, "Returns True if an edge has occured on a given GPIO.  You need to enable edge detection using add_event_detect() first.\ngpio - gpio channel"},
    {"add_event_callback", (PyCFunction)py_add_event_callback, METH_VARARGS | METH_KEYWORDS, "Add a callback for an event already defined using add_event_detect()\ngpio         - gpio channel\ncallback     - a callback function\n[bouncetime] - Switch bounce timeout in ms"},
-   {"wait_for_edge", py_wait_for_edge, METH_VARARGS, "Wait for an edge.\ngpio - gpio channel\nedge - RISING, FALLING or BOTH"},
+   {"wait_for_edge", py_wait_for_edge, METH_VARARGS, "Wait for an edge.\ngpio - gpio channel\nedge - RISING, FALLING or BOTH\ntimeout (optional) - time to wait in miliseconds. -1 will wait forever (default)"},
    {"gpio_function", py_gpio_function, METH_VARARGS, "Return the current GPIO function (IN, OUT, ALT0)\ngpio - gpio channel"},
    {"setwarnings", py_setwarnings, METH_VARARGS, "Enable or disable warning messages"},
    {NULL, NULL, 0, NULL}
@@ -517,6 +582,8 @@ PyMODINIT_FUNC initGPIO(void)
 
    define_constants(module);
 
+   initlog(LOG_NOTICE, NULL, BBIO_LOG_OPTION);
+
    if (!PyEval_ThreadsInitialized())
       PyEval_InitThreads();
 
