@@ -29,7 +29,7 @@ SOFTWARE.
 const char *valid_uarts[4] = {"UART1", "UART2", "UART4", "UART5"};
 
 // python function cleanup()
-static PyObject *py_cleanup(PyObject *self, PyObject *args)
+static PyObject *py_cleanup(__attribute__ ((unused)) PyObject *self, __attribute__ ((unused)) PyObject *args)
 {
     // unexport the UART
     uart_cleanup();
@@ -38,22 +38,25 @@ static PyObject *py_cleanup(PyObject *self, PyObject *args)
 }
 
 // python function setup()
-static PyObject *py_setup_uart(PyObject *self, PyObject *args)
+static PyObject *py_setup_uart(__attribute__ ((unused)) PyObject *self, PyObject *args)
 {
-    char dt[9];
+    char dt[FILENAME_BUFFER_SIZE];
     char *channel;
+    BBIO_err err;
 
     if (!PyArg_ParseTuple(args, "s", &channel)) {
         PyErr_SetString(PyExc_ValueError, "Invalid UART channel.");
         return NULL;
     }
 
-    if (!get_uart_device_tree_name(channel, dt)) {
+    err = get_uart_device_tree_name(channel, dt);
+    if (err != BBIO_OK) {
         PyErr_SetString(PyExc_ValueError, "Invalid UART channel.");
         return NULL;
     }
 
-    if (!uart_setup(dt)) {
+    err = uart_setup(dt);
+    if (err != BBIO_OK) {
         PyErr_SetString(PyExc_RuntimeError, "Unable to export UART channel.");
         return NULL;        
     }
@@ -104,4 +107,4 @@ PyMODINIT_FUNC initUART(void)
 #else
     return;
 #endif
-}
\ No newline at end of file
+}
