@@ -33,6 +33,13 @@
 #include <sys/ioctl.h>
 #include "common.h"
 
+#if PY_MAJOR_VERSION < 3
+#	define PyLong_AS_LONG(val) PyInt_AS_LONG(val)
+#	define PyLong_AsLong(val) PyInt_AsLong(val)
+#	undef  PyLong_Check
+#	define PyLong_Check(val) PyInt_Check(val)
+#endif
+
 PyDoc_STRVAR(SPI_module_doc,
 	"This module defines an object type that allows SPI transactions\n"
 	"on hosts running the Linux kernel. The host kernel must have SPI\n"
@@ -53,7 +60,7 @@ typedef struct {
 } SPI;
 
 static PyObject *
-SPI_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
+SPI_new(PyTypeObject *type, __attribute__ ((unused)) PyObject *args, __attribute__ ((unused)) PyObject *kwds)
 {
 	SPI *self;
 	if ((self = (SPI *)type->tp_alloc(type, 0)) == NULL)
@@ -94,7 +101,7 @@ SPI_dealloc(SPI *self)
 	PyObject *ref = SPI_close(self);
 	Py_XDECREF(ref);
 
-	self->ob_type->tp_free((PyObject *)self);
+	Py_TYPE(self)->tp_free((PyObject *)self);
 }
 
 #define MAXPATH 16
@@ -112,7 +119,7 @@ static PyObject *
 SPI_writebytes(SPI *self, PyObject *args)
 {
 	int		status;
-	uint8_t	ii, len;
+	uint16_t	ii, len;
 	uint8_t	buf[MAXMSGLEN];
 	PyObject	*list;
 
@@ -131,11 +138,11 @@ SPI_writebytes(SPI *self, PyObject *args)
 
 	for (ii = 0; ii < len; ii++) {
 		PyObject *val = PyList_GET_ITEM(list, ii);
-		if (!PyInt_Check(val)) {
+		if (!PyLong_Check(val)) {
 			PyErr_SetString(PyExc_TypeError, wrmsg);
 			return NULL;
 		}
-		buf[ii] = (__u8)PyInt_AS_LONG(val);
+		buf[ii] = (__u8)PyLong_AS_LONG(val);
 	}
 
 	status = write(self->fd, &buf[0], len);
@@ -162,7 +169,8 @@ static PyObject *
 SPI_readbytes(SPI *self, PyObject *args)
 {
 	uint8_t	rxbuf[MAXMSGLEN];
-	int		status, len, ii;
+	ssize_t len;
+	int		status, ii;
 	PyObject	*list;
 
 	if (!PyArg_ParseTuple(args, "i:read", &len))
@@ -171,7 +179,7 @@ SPI_readbytes(SPI *self, PyObject *args)
 	/* read at least 1 byte, no more than 1024 */
 	if (len < 1)
 		len = 1;
-	else if (len > sizeof(rxbuf))
+	else if (len > (ssize_t)sizeof(rxbuf))
 		len = sizeof(rxbuf);
 
 	memset(rxbuf, 0, sizeof rxbuf);
@@ -206,7 +214,7 @@ PyDoc_STRVAR(SPI_xfer_doc,
 static PyObject *
 SPI_xfer(SPI *self, PyObject *args)
 {
-	uint8_t ii, len;
+	uint16_t ii, len;
 	int status;
 	int delay = -1;
 	//uint8_t ret = 0;
@@ -231,17 +239,20 @@ SPI_xfer(SPI *self, PyObject *args)
 		delay = 0;
 	}
 
-	xferptr = (struct spi_ioc_transfer*) malloc(sizeof(struct spi_ioc_transfer) * len);
+	xferptr = (struct spi_ioc_transfer*) calloc(len, sizeof(struct spi_ioc_transfer));
 	txbuf = malloc(sizeof(__u8) * len);
 	rxbuf = malloc(sizeof(__u8) * len);
 
 	for (ii = 0; ii < len; ii++) {
 		PyObject *val = PyList_GET_ITEM(list, ii);
-		if (!PyInt_Check(val)) {
+		if (!PyLong_Check(val)) {
+			free(txbuf);
+			free(rxbuf);
+			free(xferptr);
 			PyErr_SetString(PyExc_TypeError, wrmsg);
 			return NULL;
 		}
-		txbuf[ii] = (__u8)PyInt_AS_LONG(val);
+		txbuf[ii] = (__u8)PyLong_AS_LONG(val);
 		xferptr[ii].tx_buf = (unsigned long)&txbuf[ii];
 		xferptr[ii].rx_buf = (unsigned long)&rxbuf[ii];
 		xferptr[ii].len = 1;
@@ -252,6 +263,9 @@ SPI_xfer(SPI *self, PyObject *args)
 
 	status = ioctl(self->fd, SPI_IOC_MESSAGE(len), xferptr);
 	if (status < 0) {
+		free(txbuf);
+		free(rxbuf);
+		free(xferptr);
 		PyErr_SetFromErrno(PyExc_IOError);
 		return NULL;
 	}
@@ -270,6 +284,7 @@ SPI_xfer(SPI *self, PyObject *args)
 	free(rxbuf);
 	free(xferptr);
 
+	Py_INCREF(list);
 	return list;
 }
 
@@ -285,9 +300,9 @@ SPI_xfer2(SPI *self, PyObject *args)
 	static char *msg = "Argument must be a list of at least one, "
 				"but not more than 1024 integers";
 	int status;	
-	uint8_t ii, len;
+	uint16_t ii, len;
 	PyObject *list;
-	struct spi_ioc_transfer	xfer;
+	struct spi_ioc_transfer	xfer = {0};
 	uint8_t *txbuf, *rxbuf;
 
 	if (!PyArg_ParseTuple(args, "O:xfer2", &list))
@@ -308,11 +323,13 @@ SPI_xfer2(SPI *self, PyObject *args)
 
 	for (ii = 0; ii < len; ii++) {
 		PyObject *val = PyList_GET_ITEM(list, ii);
-		if (!PyInt_Check(val)) {
+		if (!PyLong_Check(val)) {
+			free(txbuf);
+			free(rxbuf);
 			PyErr_SetString(PyExc_TypeError, msg);
 			return NULL;
 		}
-		txbuf[ii] = (__u8)PyInt_AS_LONG(val);
+		txbuf[ii] = (__u8)PyLong_AS_LONG(val);
 	}
 
 	xfer.tx_buf = (unsigned long)txbuf;
@@ -324,6 +341,8 @@ SPI_xfer2(SPI *self, PyObject *args)
 	
 	status = ioctl(self->fd, SPI_IOC_MESSAGE(1), &xfer);
 	if (status < 0) {
+		free(txbuf);
+		free(rxbuf);
 		PyErr_SetFromErrno(PyExc_IOError);
 		return NULL;
 	}
@@ -340,6 +359,7 @@ SPI_xfer2(SPI *self, PyObject *args)
 	free(txbuf);
 	free(rxbuf);
 
+	Py_INCREF(list);
 	return list;
 }
 
@@ -361,7 +381,7 @@ static int __SPI_set_mode( int fd, __u8 mode) {
 }
 
 static PyObject *
-SPI_get_mode(SPI *self, void *closure)
+SPI_get_mode(SPI *self, __attribute__ ((unused)) void *closure)
 {
 	PyObject *result = Py_BuildValue("i", (self->mode & (SPI_CPHA | SPI_CPOL) ) );
 
@@ -369,7 +389,7 @@ SPI_get_mode(SPI *self, void *closure)
 }
 
 static PyObject *
-SPI_get_cshigh(SPI *self, void *closure)
+SPI_get_cshigh(SPI *self, __attribute__ ((unused)) void *closure)
 {
 	PyObject *result;
 
@@ -378,11 +398,12 @@ SPI_get_cshigh(SPI *self, void *closure)
 	else
 		result = Py_False;
 
+	Py_INCREF(result);
 	return result;
 }
 
 static PyObject *
-SPI_get_lsbfirst(SPI *self, void *closure)
+SPI_get_lsbfirst(SPI *self, __attribute__ ((unused)) void *closure)
 {
 	PyObject *result;
 
@@ -391,11 +412,12 @@ SPI_get_lsbfirst(SPI *self, void *closure)
 	else
 		result = Py_False;
 
+        Py_INCREF(result);
 	return result;
 }
 
 static PyObject *
-SPI_get_3wire(SPI *self, void *closure)
+SPI_get_3wire(SPI *self, __attribute__ ((unused)) void *closure)
 {
 	PyObject *result;
 
@@ -404,11 +426,12 @@ SPI_get_3wire(SPI *self, void *closure)
 	else
 		result = Py_False;
 
+        Py_INCREF(result);
 	return result;
 }
 
 static PyObject *
-SPI_get_loop(SPI *self, void *closure)
+SPI_get_loop(SPI *self, __attribute__ ((unused)) void *closure)
 {
 	PyObject *result;
 
@@ -417,12 +440,13 @@ SPI_get_loop(SPI *self, void *closure)
 	else
 		result = Py_False;
 
+        Py_INCREF(result);
 	return result;
 }
 
 
 static int
-SPI_set_mode(SPI *self, PyObject *val, void *closure)
+SPI_set_mode(SPI *self, PyObject *val, __attribute__ ((unused)) void *closure)
 {
 	uint8_t mode, tmp;
 
@@ -431,13 +455,13 @@ SPI_set_mode(SPI *self, PyObject *val, void *closure)
 			"Cannot delete attribute");
 		return -1;
 	}
-	else if (!PyInt_Check(val)) {
+	else if (!PyLong_Check(val)) {
 		PyErr_SetString(PyExc_TypeError,
 			"The mode attribute must be an integer");
 		return -1;
 	}
 
-	mode = PyInt_AsLong(val);
+	mode = PyLong_AsLong(val);
 
 	if ( mode > 3 ) {
 		PyErr_SetString(PyExc_TypeError,
@@ -457,7 +481,7 @@ SPI_set_mode(SPI *self, PyObject *val, void *closure)
 }
 
 static int
-SPI_set_cshigh(SPI *self, PyObject *val, void *closure)
+SPI_set_cshigh(SPI *self, PyObject *val, __attribute__ ((unused)) void *closure)
 {
 	uint8_t tmp;
 
@@ -485,7 +509,7 @@ SPI_set_cshigh(SPI *self, PyObject *val, void *closure)
 }
 
 static int
-SPI_set_lsbfirst(SPI *self, PyObject *val, void *closure)
+SPI_set_lsbfirst(SPI *self, PyObject *val, __attribute__ ((unused)) void *closure)
 {
 	uint8_t tmp;
 
@@ -513,7 +537,7 @@ SPI_set_lsbfirst(SPI *self, PyObject *val, void *closure)
 }
 
 static int
-SPI_set_3wire(SPI *self, PyObject *val, void *closure)
+SPI_set_3wire(SPI *self, PyObject *val, __attribute__ ((unused)) void *closure)
 {
 	uint8_t tmp;
 
@@ -541,7 +565,7 @@ SPI_set_3wire(SPI *self, PyObject *val, void *closure)
 }
 
 static int
-SPI_set_loop(SPI *self, PyObject *val, void *closure)
+SPI_set_loop(SPI *self, PyObject *val, __attribute__ ((unused)) void *closure)
 {
 	uint8_t tmp;
 
@@ -569,14 +593,14 @@ SPI_set_loop(SPI *self, PyObject *val, void *closure)
 }
 
 static PyObject *
-SPI_get_bpw(SPI *self, void *closure)
+SPI_get_bpw(SPI *self, __attribute__ ((unused)) void *closure)
 {
 	PyObject *result = Py_BuildValue("i", self->bpw);
 	return result;
 }
 
 static int
-SPI_set_bpw(SPI *self, PyObject *val, void *closure)
+SPI_set_bpw(SPI *self, PyObject *val, __attribute__ ((unused)) void *closure)
 {
 	uint8_t bits;
 
@@ -585,13 +609,13 @@ SPI_set_bpw(SPI *self, PyObject *val, void *closure)
 			"Cannot delete attribute");
 		return -1;
 	}
-	else if (!PyInt_Check(val)) {
+	else if (!PyLong_Check(val)) {
 		PyErr_SetString(PyExc_TypeError,
 			"The bpw attribute must be an integer");
 		return -1;
 	}
 
-	bits = PyInt_AsLong(val);
+	bits = PyLong_AsLong(val);
 
         if (bits < 8 || bits > 16) {
 		PyErr_SetString(PyExc_TypeError,
@@ -610,14 +634,14 @@ SPI_set_bpw(SPI *self, PyObject *val, void *closure)
 }
 
 static PyObject *
-SPI_get_msh(SPI *self, void *closure)
+SPI_get_msh(SPI *self, __attribute__ ((unused)) void *closure)
 {
 	PyObject *result = Py_BuildValue("i", self->msh);
 	return result;
 }
 
 static int
-SPI_set_msh(SPI *self, PyObject *val, void *closure)
+SPI_set_msh(SPI *self, PyObject *val, __attribute__ ((unused)) void *closure)
 {
 	uint32_t msh;
 
@@ -626,13 +650,13 @@ SPI_set_msh(SPI *self, PyObject *val, void *closure)
 			"Cannot delete attribute");
 		return -1;
 	}
-	else if (!PyInt_Check(val)) {
+	else if (!PyLong_Check(val)) {
 		PyErr_SetString(PyExc_TypeError,
 			"The msh attribute must be an integer");
 		return -1;
 	}
 
-	msh = PyInt_AsLong(val);
+	msh = PyLong_AsLong(val);
 	// DAW - 8/12/12 - removed limitation on SPI speed
 	// if (8000000 < msh) {
 		// PyErr_SetString(PyExc_TypeError,
@@ -679,6 +703,7 @@ static PyObject *
 SPI_open(SPI *self, PyObject *args, PyObject *kwds)
 {
 	int bus, device;
+	int bus_path;
 	int max_dt_length = 15;
 	char device_tree_name[max_dt_length];
 	char path[MAXPATH];
@@ -687,17 +712,24 @@ SPI_open(SPI *self, PyObject *args, PyObject *kwds)
 	static char *kwlist[] = {"bus", "device", NULL};
 	if (!PyArg_ParseTupleAndKeywords(args, kwds, "ii:open", kwlist, &bus, &device))
 		return NULL;
-	if (snprintf(device_tree_name, max_dt_length, "ADAFRUIT-SPI%d", bus) >= max_dt_length) {
+	if (snprintf(device_tree_name, max_dt_length, "BB-SPIDEV%d", bus) >= max_dt_length) {
 		PyErr_SetString(PyExc_OverflowError,
 			"Bus and/or device number is invalid.");
 		return NULL;
 	}
-	if (load_device_tree(device_tree_name) == -1) {
+	if (load_device_tree(device_tree_name) == BBIO_CAPE) {
 		PyErr_SetFromErrno(PyExc_IOError);
 		return NULL;
 	}
 
-	if (snprintf(path, MAXPATH, "/dev/spidev%d.%d", bus+1, device) >= MAXPATH) {
+	bus_path = get_spi_bus_path_number(bus);
+	if (bus_path == -1) {
+		PyErr_SetString(PyExc_OverflowError,
+			"Unable to find loaded spi bus path.");
+		return NULL;
+	}
+
+	if (snprintf(path, MAXPATH, "/dev/spidev%d.%d", bus_path, device) >= MAXPATH) {
 		PyErr_SetString(PyExc_OverflowError,
 			"Bus and/or device number is invalid.");
 		return NULL;
@@ -769,8 +801,7 @@ static PyMethodDef SPI_methods[] = {
 };
 
 static PyTypeObject SPI_type = {
-	PyObject_HEAD_INIT(NULL)
-	0,				/* ob_size */
+	PyVarObject_HEAD_INIT(NULL, 0)
 	"SPI",			/* tp_name */
 	sizeof(SPI),			/* tp_basicsize */
 	0,				/* tp_itemsize */
@@ -814,20 +845,51 @@ static PyMethodDef SPI_module_methods[] = {
 	{NULL}
 };
 
+#if PY_MAJOR_VERSION >= 3
+static struct PyModuleDef moduledef = {
+	PyModuleDef_HEAD_INIT,
+	"SPI",               /* m_name */
+	SPI_module_doc,      /* m_doc */
+	-1,                  /* m_size */
+	SPI_module_methods,  /* m_methods */
+	NULL,                /* m_reload */
+	NULL,                /* m_traverse */
+	NULL,                /* m_clear */
+	NULL,                /* m_free */
+};
+#endif
+
 #ifndef PyMODINIT_FUNC	/* declarations for DLL import/export */
 #define PyMODINIT_FUNC void
 #endif
 PyMODINIT_FUNC
+#if PY_MAJOR_VERSION >= 3
+PyInit_SPI(void)
+#else
 initSPI(void)
+#endif
 {
 	PyObject* m;
 
 	if (PyType_Ready(&SPI_type) < 0)
+		#if PY_MAJOR_VERSION >= 3
+		return NULL;
+		#else
 		return;
+		#endif
 
+#if PY_MAJOR_VERSION >= 3
+	m = PyModule_Create(&moduledef);
+#else
 	m = Py_InitModule3("SPI", SPI_module_methods, SPI_module_doc);
+#endif
+
 	Py_INCREF(&SPI_type);
 	PyModule_AddObject(m, "SPI", (PyObject *)&SPI_type);
+
+#if PY_MAJOR_VERSION >= 3
+	return m;
+#endif
 }
 
 
