@@ -1,11 +1,16 @@
 import pytest
 import os
+import platform
 
 import Adafruit_BBIO.GPIO as GPIO
 
+kernel = platform.release()
+
+
 def teardown_module(module):
     GPIO.cleanup()
 
+
 class TestSetup:
     def test_setup_output_key(self):
         GPIO.setup("P8_10", GPIO.OUT)
@@ -15,7 +20,11 @@ class TestSetup:
         GPIO.cleanup()
 
     def test_setup_output_name(self):
-        GPIO.setup("TIMER6", GPIO.OUT)
+        # WARNING: TIMERn syntax is not working on newer kernels
+        #          such as 4.4. Originally discovered while testing
+        #          Pull Request #152. See issue #156 for details.
+        #GPIO.setup("TIMER6", GPIO.OUT)
+        GPIO.setup("P8_10", GPIO.OUT)
         assert os.path.exists('/sys/class/gpio/gpio68')
         direction = open('/sys/class/gpio/gpio68/direction').read()
         assert direction == 'out\n'        
@@ -29,7 +38,11 @@ class TestSetup:
         GPIO.cleanup()
 
     def test_setup_input_name(self):
-        GPIO.setup("TIMER6", GPIO.IN)
+        # WARNING: TIMERn syntax is not working on newer kernels
+        #          such as 4.4. Originally discovered while testing
+        #          Pull Request #152. See issue #156 for details.
+        #GPIO.setup("TIMER6", GPIO.IN)
+        GPIO.setup("P8_10", GPIO.IN)
         assert os.path.exists('/sys/class/gpio/gpio68')
         direction = open('/sys/class/gpio/gpio68/direction').read()
         assert direction == 'in\n'        
@@ -53,7 +66,10 @@ class TestSetup:
         GPIO.setup("P8_10", GPIO.OUT)
         assert os.path.exists('/sys/class/gpio/gpio68')
         GPIO.cleanup()
-        assert not os.path.exists('/sys/class/gpio/gpio68')
+        if kernel < '4.1.0':
+            assert not os.path.exists('/sys/class/gpio/gpio68')
+            # for later kernels, the universal capemanager always loads the
+            # UARTs.
 
     def test_setup_failed_type_error(self):
         with pytest.raises(TypeError):
@@ -69,4 +85,7 @@ class TestSetup:
         GPIO.setup("P9_31", GPIO.OUT)
         assert os.path.exists('/sys/class/gpio/gpio110')
         GPIO.cleanup()
-        assert not os.path.exists('/sys/class/gpio/gpio110')
+        if kernel < '4.1.0':
+            assert not os.path.exists('/sys/class/gpio/gpio110')
+            # for later kernels, the universal capemanager always loads the
+            # UARTs.
